<?php

namespace Drupal\Tests\fieldable_path\Functional;

use Drupal\field\Entity\FieldConfig;
use Drupal\simpletest\WebTestBase;
use Drupal\field\Entity\FieldStorageConfig;

/**
 * Provides basic test coverage for CRUD functionality.
 *
 * @group fieldable_path
 */
class FieldablePathBasicTest extends WebTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'path',
    'fieldable_path',
    'node',
  ];

  /**
   * @var \Drupal\user\Entity\User
   */
  protected $user;

  /**
   * @var string
   */
  protected $field_name = 'field_path';

  /**
   * @var string
   */
  protected $field_type = 'fieldable_path';

  /**
   * @var string
   */
  protected $field_widget = 'fieldable_path_widget';

  protected function setUp() {
    parent::setUp();

    // Create a new Article node type.
    $this->drupalCreateContentType([
      'type' => 'article',
      'name' => 'Article',
    ]);

    FieldStorageConfig::create([
      'entity_type' => 'node',
      'field_name' => $this->field_name,
      'type' => $this->field_type,
      'settings' => [],
      'cardinality' => 1,
    ])->save();


    // Add field to the article node type.
    FieldConfig::create([
      'entity_type' => 'node',
      'bundle' => 'article',
      'field_name' => $this->field_name,
      'label' => 'Path',
      'required' => FALSE,
      'settings' => [],
      'description' => '',
    ])->save();

    // Add the field to the node article form.
    entity_get_form_display('node', 'article', 'default')
      ->setComponent($this->field_name, [
        'type' => $this->field_widget
      ])
      ->save();

    /*entity_get_display('node', 'article', 'full')
      ->setComponent($this->field_name)
      ->save();*/

    $this->user = $this->drupalCreateUser([
      'create article content',
      'edit any article content',
      'create url aliases',
    ]);

    $this->drupalLogin($this->user);
  }

  public function testNodeCRUD() {

    /**
     * Test creation of a new node with new path alias.
     */

    $pathAlias = '/' . $this->randomMachineName();
    $edit = [
      'title[0][value]' => $this->randomMachineName(),
      'path[0][alias]' => $pathAlias,
    ];

    // Create a new node with path alias set.
    $this->drupalPostForm('node/add/article', $edit, t('Save'));

    // Get node edit url from the page header.
    $xpath = $this->xpath('//link[@rel=\'edit-form\']');
    $editPage = (string) $xpath[0]->attributes()->href;

    // Make sure path field got the right value.
    $this->drupalGet($editPage);
    $this->assertFieldByName($this->field_name . '[0][value]', $pathAlias);

    // Make sure field is disabled.
    $xpath = $this->xpath('//input[@name=\'' . $this->field_name . '[0][value]\']');
    $disabledValue = (string) $xpath[0]->attributes()->disabled;
    $this->assertEqual($disabledValue, 'disabled');

    /**
     * Test updating of path alias.
     */

    // Set the new value to the path alias.
    $pathAlias = '/' . $this->randomMachineName();
    $edit = [ 'path[0][alias]' => $pathAlias ];
    $this->drupalPostForm($editPage, $edit, t('Save'));

    // Make sure path field got the right value.
    $this->drupalGet($editPage);
    $this->assertFieldByName($this->field_name . '[0][value]', $pathAlias);

    /**
     * Test deletion of path alias.
     */

    // Set the new value to the path alias.
    $edit = [ 'path[0][alias]' => '' ];
    $this->drupalPostForm($editPage, $edit, t('Save'));

    // Make sure path field has empty value.
    $this->drupalGet($editPage);
    $this->assertFieldByName($this->field_name . '[0][value]', '');
  }
}
